#ifndef BACK_LIGHT_APP
#define BACK_LIGHT_APP

#include "TYW_stdint.h"

extern void BL_Init(void);
extern void BL_TFT_Start(void);
extern void BL_LCD_Start(void);
extern void BL_TFT_Stop(void);
extern void BL_LCD_Stop(void);
extern void BL_Sleep(void);
extern void BL_KL30_Init(void);
extern void DesNitBrt_Add(void);
extern void DesDayBrt_Add(void);
extern void DesNitBrt_Sub(void);
extern void DesDayBrt_Sub(void);
extern void Brightness_Mode_Set(uint8_t mode);

extern uint8_t DesNitBrt_Get(void);
extern uint8_t CurNitBrt_Get(void);
extern uint8_t DesDayBrt_Get(void);
extern uint8_t CurDayBrt_Get(void);

extern void BL_Open_Flag_Set(uint8_t val);
extern void BL_Management_service(void);

#endif